import platform
from typing import Dict
from werkflow_core import Module
from .architecture.types import ArchitectureType
from .configuration import Configuration
from .exceptions import (
    UnsupportedArchitectureException,
    UnsupportedOperatingSystemException
)
from .operating_system import (
    Linux,
    MacOS,
    Windows,
    OperatingSystem,
    OperatingSystemType
)
from .users import Users


class System(Module):

    def __init__(self) -> None:
        super().__init__()

        self._supported_operating_systems: Dict[OperatingSystemType, OperatingSystem] = {
            OperatingSystemType.LINUX: Linux,
            OperatingSystemType.MAC_OS: MacOS,
            OperatingSystemType.WINDOWS: Windows
        }

        self._supported_architectures: Dict[str, ArchitectureType] = {
            architecture.value: architecture for architecture in ArchitectureType
        }

        self.users = Users()
        
        operating_system_type = OperatingSystem.detect()
        self.os: OperatingSystem = None

        try:

            self.os: OperatingSystem = self._supported_operating_systems[operating_system_type]()

        except KeyError:
            raise UnsupportedOperatingSystemException(
                list([
                    os_option.value for os_option in self._supported_operating_systems.keys()
                ])
            )
           
        processor_architecture = self._supported_architectures.get(
            platform.machine()
        )
        
        self.architecture = self.os.supported_architectures.get(processor_architecture)

        if self.architecture is None:
            raise UnsupportedArchitectureException(
                self.os.name,
                processor_architecture,
                list(self.os.supported_architectures.values()),
            )
        
        self.configuration = Configuration()



import platform

from werkflow_system.architecture.types import ArchitectureType

from .base import OperatingSystem
from .types import OperatingSystemType


class MacOS(OperatingSystem):

    def __init__(self) -> None:
        super().__init__()

        self.name = 'MacOS'
        self.type = OperatingSystemType.MAC_OS

        self.supported_architectures = {
            ArchitectureType.ARM_64: 'arm64',
            ArchitectureType.x86_64: 'x86_64',
            ArchitectureType.AARCH64: 'aarch64',
        }
    
        mac_os_version, _, _ = platform.mac_ver()
        
        self.version = mac_os_version



import os
import psutil
from getpass import getuser
from typing import Iterable
from pathlib import Path


class Users:

    def __init__(self) -> None:
                
        self._system_users = psutil.users()
        
        self.current_user = getuser()
        if self.current_user is None:
            self.current_user = os.getlogin()

        self.home= Path.home()

    def list(self) -> Iterable[str]:
        for user in self._system_users:
            if user.name.startswith('_') is False:
                yield user


class DataAmount:

    def __init__(self, value: int) -> None:
        self.value: int = value

    @property
    def kb(self) -> float:
        return round(self.value/10**3, 2)

    @property
    def mb(self) -> float:
        return round(self.value/10**6, 2)
    
    @property
    def gb(self) -> float:
        return round(self.value/10**9, 2)
    
    @property
    def tb(self) -> float:
        return round(self.value/10**12, 2)


from typing import List


class MissingVolumeException(Exception):

    def __init__(
        self,
        supported_drives: List[str]
    ) -> None:
        
        supported_drive_names = '-\n'.join(supported_drives)

        super().__init__(
            f'Drive does not exist or has not been mounted.\nFound drives include:\n{supported_drive_names}'
        )


import psutil


class CPU:

    def __init__(self) -> None:
        self.physical = psutil.cpu_count(logical=False)
        self.total = psutil.cpu_count(logical=True)
        self.percentage_used = psutil.cpu_percent()

    def update(self):
        self.percentage_used = psutil.cpu_percent()



import platform
import psutil
from typing import Dict
from werkflow_system.architecture.types import ArchitectureType
from .types import OperatingSystemType


class OperatingSystem:

    def __init__(self) -> None:
        self.name: str = None
        self.type: OperatingSystemType = None
        self.version = platform.release()
        self.supported_architectures: Dict[ArchitectureType, str] = {}

    @classmethod
    def detect(cls) -> OperatingSystemType:
        
        if psutil.MACOS:
            return OperatingSystemType.MAC_OS
        
        elif psutil.WINDOWS:
            return OperatingSystemType.WINDOWS
        
        else:
            return OperatingSystemType.LINUX




import platform
from typing import List


class UnsupportedOperatingSystemException(Exception):

    def __init__(
        self, 
        supported_os_options: List[str]
    ) -> None:

        supported_options = ', '.join(supported_os_options)
        detected_os = platform.system()

        super().__init__(
            f'Unsupported OS.\n -Found: {detected_os}\n- Supports: {supported_options}'
        )


from typing import Dict, List, Union
from .cpu import CPU
from .drive_group import DriveGroup
from .memory import Memory

CPUStats = Dict[str, Union[int, float]]
MemoryStats = Dict[str, Union[int, float]]
DiskStats = List[Dict[str, Union[str, int, float]]]

ConfigurationSummaryDict = Dict[str, Union[CPUStats, MemoryStats, DiskStats]]

class ConfigurationSummary:

    def __init__(
        self,
        cpu: CPU,
        drives: DriveGroup,
        memory: Memory
    ) -> None:
        
        cpu.update()
        self.cpu = {
            'cpu_physical': cpu.physical,
            'cpu_total': cpu.total,
            'cpu_percentage_used': cpu.percentage_used
        }

        memory.update()
        self.memory = {
            'memory_total_mb': memory.total.mb,
            'memory_used_mb': memory.used.mb,
            'memory_available_mb': memory.available.mb,
            'memory_percentage_used': memory.percentage_used
        }

        drives.update()
        self.drives = [
            {
                'drive_name': drive.name,
                'drive_path': drive.path,
                'drive_options': drive.options,
                'drive_total_mb': drive.total.mb,
                'drive_used_mb': drive.used.mb,
                'drive_available_mb': drive.available.mb,
                'drive_percentage_used': drive.percentage_used

            } for drive in drives
        ]

    def to_dict(self) -> ConfigurationSummaryDict:
        return {
            'cpu': self.cpu,
            'memory': self.memory,
            'drives': self.drives
        }


from .cpu import CPU
from .drive_group import DriveGroup
from .memory import Memory
from .configuration_summary import ConfigurationSummary


class Configuration:

    def __init__(self) -> None:
        self.cores = CPU()
        self.drives = DriveGroup()
        self.memory = Memory()

    def summary(self) -> ConfigurationSummary:
        return ConfigurationSummary(
            self.cores,
            self.drives,
            self.memory
        )